%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% networksolve
% Carlos J. Cela, 2012
%
% Given a network of resistances and current sources,
% creates a linear system and solves it using KCL. Node 
% number 0 is assumed to be ground. Node numbers are natural
% numbers, without skipping.
%
% The parameters are as follow:
%
% r   Resistance matrix, having 3 columns, and one row
%     per resistance element.
%         1st column: node A number
%         2nd column: node B number
%         3rd column: R value in Ohms
%
% i   Current matrix, having 3 columns, and one row
%     per current source.
%         1st column: node A (+) number
%         2nd column: node B (-) number
%         3rd column: I value in Amperes
%
% Return value is:
% v   Voltage vector, having the voltages in Volts 
%     for each node.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [v] = networksolve( r, i)

% Determine the number of nodes
s = max([max(r(:,1)) max(r(:,2)) max(i(:,1)) max(r(:,2))]);

% Create G matrix and current vector I
  G(1:s,1:s) = 0;
  I(1:s) = 0;

% Determine number of resistors
nr = size(r); nr = nr(1);

% Fill admittance matrix G
for n = 1:nr
  a = r(n,1);
  b = r(n,2);
  rn = r(n,3);

  % Diagonal
  if a!=0
    G(a,a) = G(a,a) + 1/rn;
  end

  if b!=0
    G(b,b) = G(b,b) + 1/rn;
  end

  if a!=0 && b!=0
    G(a,b) = G(a,b) - 1/rn;
    G(b,a) = G(a,b);
  end
end

% Determine number of current sources
ni = size(i); ni = ni(1);

% Fill current vector I
for n = 1:ni
  a = i(n,1);
  b = i(n,2);
  in = i(n,3);

  if a==0
    I(b) = I(b) - in;
  end

  if b==0
    I(a) = I(a) + in;
  end

  if a!=0 && b!=0
    I(a) = I(a) + in; 
    I(b) = I(b) - in;  
  end
end
I =I';

% Solve
v = G\I;


end
